@@ -183,7 +183,7 @@ class Extractor():
         elif until:
             if isinstance(until, datetime.datetime):
                 # convert to UTC timestamp
-                until = (until - util.EPOCH) / util.SECOND
+                until = util.datetime_to_timestamp(until)
             else:
                 until = float(until)
             seconds = until - now
@@ -373,7 +373,6 @@ class Extractor():
                         self.log.warning(
                             "Cookie '%s' will expire in less than %s hour%s",
                             cookie.name, hours + 1, "s" if hours else "")
-                        continue
 
                 names.discard(cookie.name)
                 if not names:
@@ -604,18 +603,21 @@ class BaseExtractor(Extractor):
 
     def __init__(self, match):
         if not self.category:
-            for index, group in enumerate(match.groups()):
-                if group is not None:
-                    if index:
-                        self.category, self.root = self.instances[index-1]
-                        if not self.root:
-                            self.root = text.root_from_url(match.group(0))
-                    else:
-                        self.root = group
-                        self.category = group.partition("://")[2]
-                    break
+            self._init_category(match)
         Extractor.__init__(self, match)
 
+    def _init_category(self, match):
+        for index, group in enumerate(match.groups()):
+            if group is not None:
+                if index:
+                    self.category, self.root = self.instances[index-1]
+                    if not self.root:
+                        self.root = text.root_from_url(match.group(0))
+                else:
+                    self.root = group
+                    self.category = group.partition("://")[2]
+                break
+
     @classmethod
     def update(cls, instances):
         extra_instances = config.get(("extractor",), cls.basecategory)
