@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2019 Mike Fährmann
+# Copyright 2015-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extract images from https://nijie.info/"""
+"""Extractors for https://nijie.info/"""
 
 from .common import Extractor, Message, AsynchronousMixin
 from .. import text, exception
@@ -36,7 +36,6 @@ class NijieExtractor(AsynchronousMixin, Extractor):
 
     def items(self):
         self.login()
-        yield Message.Version, 1
 
         for image_id in self.image_ids():
 
@@ -69,8 +68,8 @@ class NijieExtractor(AsynchronousMixin, Extractor):
             "description": text.unescape(extr(
                 '"description": "', '"').replace("&amp;", "&")),
             "date"       : text.parse_datetime(extr(
-                '"datePublished": "', '"')[:-4] + "+0900",
-                "%a %d %b %Y %I:%M:%S %p%z"),
+                '"datePublished": "', '"') + "+0900",
+                "%a %b %d %H:%M:%S %Y%z"),
             "artist_id"  : text.parse_int(extr(
                 '"sameAs": "https://nijie.info/members.php?id=', '"')),
             "artist_name": keywords[1],
@@ -100,6 +99,10 @@ class NijieExtractor(AsynchronousMixin, Extractor):
 
     @cache(maxage=150*24*3600, keyarg=1)
     def _login_impl(self, username, password):
+        if not username or not password:
+            raise exception.AuthenticationError(
+                "Username and password required")
+
         self.log.info("Logging in as %s", username)
         url = "{}/login_int.php".format(self.root)
         data = {"email": username, "password": password, "save": "on"}
@@ -127,12 +130,28 @@ class NijieExtractor(AsynchronousMixin, Extractor):
 
 
 class NijieUserExtractor(NijieExtractor):
-    """Extractor for works of a nijie-user"""
+    """Extractor for nijie user profiles"""
     subcategory = "user"
-    pattern = BASE_PATTERN + r"/members(?:_illust)?\.php\?id=(\d+)"
+    cookiedomain = None
+    pattern = BASE_PATTERN + r"/members\.php\?id=(\d+)"
+    test = ("https://nijie.info/members.php?id=44",)
+
+    def items(self):
+        base = "{}/{{}}.php?id={}".format(self.root, self.user_id)
+        return self._dispatch_extractors((
+            (NijieIllustrationExtractor, base.format("members_illust")),
+            (NijieDoujinExtractor      , base.format("members_dojin")),
+            (NijieFavoriteExtractor    , base.format("user_like_illust_view")),
+        ), ("illustration", "doujin"))
+
+
+class NijieIllustrationExtractor(NijieExtractor):
+    """Extractor for all illustrations of a nijie-user"""
+    subcategory = "illustration"
+    pattern = BASE_PATTERN + r"/members_illust\.php\?id=(\d+)"
     test = (
         ("https://nijie.info/members_illust.php?id=44", {
-            "url": "66c4ff94c6e77c0765dd88f2d8c663055fda573e",
+            "url": "1553e5144df50a676f5947d02469299b401ad6c0",
             "keyword": {
                 "artist_id": 44,
                 "artist_name": "ED",
@@ -144,7 +163,7 @@ class NijieUserExtractor(NijieExtractor):
                 "num": int,
                 "tags": list,
                 "title": str,
-                "url": r"re:https://pic.nijie.net/\d+/nijie_picture/.*jpg$",
+                "url": r"re:https://pic.nijie.net/\d+/nijie/.*jpg$",
                 "user_id": 44,
                 "user_name": "ED",
             },
@@ -152,7 +171,6 @@ class NijieUserExtractor(NijieExtractor):
         ("https://nijie.info/members_illust.php?id=43", {
             "exception": exception.NotFoundError,
         }),
-        ("https://nijie.info/members.php?id=44"),
     )
 
     def image_ids(self):
@@ -205,8 +223,8 @@ class NijieImageExtractor(NijieExtractor):
     pattern = BASE_PATTERN + r"/view(?:_popup)?\.php\?id=(\d+)"
     test = (
         ("https://nijie.info/view.php?id=70720", {
-            "url": "5497f897311397dafa188521258624346a0af2a3",
-            "keyword": "fd12bca6f4402a0c996315d28c65f7914ad70c51",
+            "url": "3d654e890212ba823c9647754767336aebc0a743",
+            "keyword": "41da5d0e178b04f01fe72460185df52fadc3c91b",
             "content": "d85e3ea896ed5e4da0bca2390ad310a4df716ca6",
         }),
         ("https://nijie.info/view.php?id=70724", {
