@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2021 Mike Fährmann
+# Copyright 2015-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -188,6 +188,10 @@ class TestCookiesTxt(unittest.TestCase):
             ".example.org\tTRUE\t/\tTRUE\t\tname\t",
             [self._cookie("name", "", ".example.org")],
         )
+        _assert(
+            "\tTRUE\t/\tTRUE\t\tname\t",
+            [self._cookie("name", "", "")],
+        )
         _assert(
             "# Netscape HTTP Cookie File\n"
             "\n"
@@ -241,6 +245,8 @@ class TestCookiesTxt(unittest.TestCase):
                     "n4", ""  , "www.example.org", False, "/", False),
                 self._cookie(
                     "n5", "v5", "www.example.org", False, "/path", False, 100),
+                self._cookie(
+                    "n6", "v6", "", False),
             ],
             "# Netscape HTTP Cookie File\n"
             "\n"
@@ -313,6 +319,27 @@ class TestOther(unittest.TestCase):
         self.assertSequenceEqual(
             list(util.unique_sequence([1, 2, 1, 3, 2, 1])), [1, 2, 1, 3, 2, 1])
 
+    def test_contains(self):
+        c = [1, "2", 3, 4, "5", "foo"]
+        self.assertTrue(util.contains(c, 1))
+        self.assertTrue(util.contains(c, "foo"))
+        self.assertTrue(util.contains(c, [1, 3, "5"]))
+        self.assertTrue(util.contains(c, ["a", "b", "5"]))
+        self.assertFalse(util.contains(c, "bar"))
+        self.assertFalse(util.contains(c, [2, 5, "bar"]))
+
+        s = "1 2 3 asd qwe y(+)c f(+)(-) bar"
+        self.assertTrue(util.contains(s, "y(+)c"))
+        self.assertTrue(util.contains(s, ["asd", "qwe", "yxc"]))
+        self.assertTrue(util.contains(s, ["sdf", "dfg", "qwe"]))
+        self.assertFalse(util.contains(s, "tag1"))
+        self.assertFalse(util.contains(s, ["tag1", "tag2", "tag3"]))
+
+        s = "1, 2, 3, asd, qwe, y(+)c, f(+)(-), bar"
+        self.assertTrue(util.contains(s, "y(+)c", ", "))
+        self.assertTrue(util.contains(s, ["sdf", "dfg", "qwe"], ", "))
+        self.assertFalse(util.contains(s, "tag1", ", "))
+
     def test_raises(self):
         func = util.raises(Exception)
         with self.assertRaises(Exception):
@@ -357,6 +384,35 @@ class TestOther(unittest.TestCase):
         with self.assertRaises(exception.StopExtraction):
             expr()
 
+    def test_build_duration_func(self, f=util.build_duration_func):
+
+        def test_single(df, v):
+            for _ in range(10):
+                self.assertEqual(df(), v)
+
+        def test_range(df, lower, upper):
+            for __ in range(10):
+                v = df()
+                self.assertGreaterEqual(v, lower)
+                self.assertLessEqual(v, upper)
+
+        for v in (0, 0.0, "", None, (), []):
+            self.assertIsNone(f(v))
+
+        for v in (0, 0.0, "", None, (), []):
+            test_single(f(v, 1.0), 1.0)
+
+        test_single(f(3), 3)
+        test_single(f(3.0), 3.0)
+        test_single(f("3"), 3)
+        test_single(f("3.0-"), 3)
+        test_single(f("  3  -"), 3)
+
+        test_range(f((2, 4)), 2, 4)
+        test_range(f([2, 4]), 2, 4)
+        test_range(f("2-4"), 2, 4)
+        test_range(f("  2.0  - 4 "), 2, 4)
+
     def test_extractor_filter(self):
         # empty
         func = util.build_extractor_filter("")
@@ -506,7 +562,16 @@ class TestOther(unittest.TestCase):
         self.assertEqual(f(["a", "b", "c"]), "a, b, c")
         self.assertEqual(f([1, 2, 3]), "1, 2, 3")
 
-    def test_to_timestamp(self, f=util.to_timestamp):
+    def test_datetime_to_timestamp(self, f=util.datetime_to_timestamp):
+        self.assertEqual(f(util.EPOCH), 0.0)
+        self.assertEqual(f(datetime.datetime(2010, 1, 1)), 1262304000.0)
+        self.assertEqual(f(datetime.datetime(2010, 1, 1, 0, 0, 0, 128000)),
+                         1262304000.128000)
+        with self.assertRaises(TypeError):
+            f(None)
+
+    def test_datetime_to_timestamp_string(
+            self, f=util.datetime_to_timestamp_string):
         self.assertEqual(f(util.EPOCH), "0")
         self.assertEqual(f(datetime.datetime(2010, 1, 1)), "1262304000")
         self.assertEqual(f(None), "")
